from copy import copy

import hpccm
from hpccm import linux_distro
from hpccm.building_blocks import (
    openmpi,
    generic_autotools,
    pmix,
    slurm_pmi2,
    ucx,
    gdrcopy,
    knem,
    multi_ofed,
    mlnx_ofed,
    packages,
)
from hpccm.primitives import comment, copy, label, shell, environment, runscript

print(f"Evaluating {filename}")

# Add cli arguments to args_parser
parse_g = parser.add_argument_group(filename)
parse_g.add_argument(
    "--ompi",
    type=str,
    default="4.0.6",
    help="OpenMPI version, e.g. 2.1.1, 2.1.5, 3.0.1, 3.1.2",
)
parse_g.add_argument(
    "--mpi_benchmarks",
    dest="mpi_benchmarks",
    action="store_true",
    help="Installs OSU MPI benchmarks and mpi_bw, mpi_ring, mpi_hello",
)
parse_g.add_argument(
    "--mpi_no_entrypoint",
    dest="mpi_no_entrypoint",
    action="store_true",
    help="Disables mpi entrypoint. (Use with ogs_jupyter.py recipe)",
)
# parse_g.add_argument(
#     "--ucx",
#     type=str,
#     default="1.8.1",
#     help="UCX version",
# )
# parse_g.add_argument(
#     "--slurm",
#     type=str,
#     default="20.02.5",
#     help="Slurm version",
# )
# parse_g.add_argument(
#     "--infiniband",
#     dest="infiniband",
#     action="store_true",
#     help="Enable Infiniband",
# )

# Parse local args
local_args = parser.parse_known_args()[0]

# set image file name
img_file += f"-openmpi-{local_args.ompi}"

# Optionally set out_dir
out_dir += f"/openmpi/{local_args.ompi}"

# Implement recipe
Stage0 += comment(f"--- Begin {filename} ---")

mpicc = ""

# Begin copy from https://github.com/NVIDIA/hpc-container-maker/blob/master/recipes/osu_benchmarks/common.py
if hpccm.config.g_linux_distro == linux_distro.CENTOS:
    Stage0 += shell(commands=["ln -s /usr/bin/gcc /usr/bin/cc"])  # Fix for gdrcopy
    Stage0 += gdrcopy(ldconfig=True)
    Stage0 += knem(ldconfig=True)

    # Mellanox legacy OFED support
    mlnx_versions = ["4.6-1.0.1.1", "4.7-3.2.9.0", "4.9-0.1.7.0", "4.9-2.2.4.0"]
    Stage0 += multi_ofed(
        inbox=False,
        mlnx_versions=mlnx_versions,
        prefix="/usr/local/ofed",
        symlink=False,
    )

    # RDMA-core based OFED support
    Stage0 += mlnx_ofed(version="5.1-0.6.6.0", symlink=False)

    # UCX default - RDMA-core based OFED
    Stage0 += ucx(
        version="1.9.0",
        cuda=False,
        gdrcopy="/usr/local/gdrcopy",
        knem="/usr/local/knem",
        disable_static=True,
        enable_mt=True,
    )

    # UCX - Mellanox legacy support
    Stage0 += ucx(
        version="1.9.0",
        build_environment={
            "LD_LIBRARY_PATH": "/usr/local/ofed/4.6-1.0.1.1/lib:${LD_LIBRARY_PATH}"
        },
        cuda=False,
        environment=False,
        gdrcopy="/usr/local/gdrcopy",
        knem="/usr/local/knem",
        prefix="/usr/local/ucx-mlnx-legacy",
        disable_static=True,
        enable_mt=True,
        with_verbs="/usr/local/ofed/4.6-1.0.1.1/usr",
        with_rdmacm="/usr/local/ofed/4.6-1.0.1.1/usr",
    )

    # Symlink legacy UCX into legacy OFED versions
    Stage0 += shell(
        commands=[
            "ln -s /usr/local/ucx-mlnx-legacy/{1}/* /usr/local/ofed/{0}/usr/{1}".format(
                version, directory
            )
            for version in mlnx_versions
            for directory in ["bin", "lib"]
        ]
    )

    # PMI support
    Stage0 += slurm_pmi2(prefix="/usr/local/pmi", version="20.11.7")
    Stage0 += pmix()

    # OpenMPI
    mpicc = openmpi(
        cuda=False,
        infiniband=False,
        ldconfig=True,
        ucx=True,
        version=local_args.ompi,
        disable_oshmem=True,
        disable_static=True,
        enable_mca_no_build="btl-uct",
        with_slurm=False,
        with_pmi="/usr/local/pmi",
        with_pmix="/usr/local/pmix",
    )
    Stage1 += environment(
        variables={
            "OMPI_MCA_pml": "ucx",
            "UCX_TLS": "all",
        }
    )
else:
    mpicc = openmpi(
        cuda=False,
        infiniband=False,
        ldconfig=True,
        version=local_args.ompi,
        disable_oshmem=True,
        disable_static=True,
        enable_mca_no_build="btl-uct",
        with_slurm=True,
    )
toolchain = mpicc.toolchain
Stage0 += mpicc

# OpenMPI expects this program to exist, even if it's not used.
# Default is "ssh : rsh", but that's not installed.
# TODO: /usr/etc/openmpi-mca-params.conf ?
# Stage0 += shell(
#     commands=[
#         "mkdir /mnt/0",
#         "echo 'plm_rsh_agent = false' >> /mnt/0/openmpi-mca-params.conf",
#         "echo 'mpi_warn_on_fork = 0' >> /mnt/0/openmpi-mca-params.conf",
#         "echo 'btl_openib_warn_default_gid_prefix = 0' >> /mnt/0/openmpi-mca-params.conf",
#     ]
# )

Stage0 += label(
    metadata={
        "org.opengeosys.mpi": "openmpi",
        "org.opengeosys.mpi.version": local_args.ompi,
        # "org.opengeosys.ucx": local_args.ucx,
        # "org.opengeosys.slurm": local_args.slurm,
        # "org.opengeosys.pmix": pmix_version,
    }
)

# Allow running MPI as root
Stage1 += environment(
    variables={"OMPI_ALLOW_RUN_AS_ROOT": "1", "OMPI_ALLOW_RUN_AS_ROOT_CONFIRM": "1"}
)

# Entrypoint
if not local_args.mpi_no_entrypoint:
    Stage1 += packages(ospackages=["curl", "ca-certificates"])
    Stage1 += shell(
        commands=[
            "curl -o /usr/local/bin/entrypoint.sh https://gitlab.opengeosys.org/ogs/container-maker/-/raw/main/ogscm/recipes/mpi-entrypoint.sh"
        ]
    )
    Stage1 += runscript(commands=["/usr/local/bin/entrypoint.sh"])

# Performance and compatibility tuning
Stage1 += environment(
    variables={
        "CUDA_CACHE_DISABLE": "1",
        "MELLANOX_VISIBLE_DEVICES": "all",  # enroot
    }
)

if local_args.mpi_benchmarks:
    Stage0 += generic_autotools(
        build_environment={"CC": "mpicc", "CXX": "mpicxx"},
        # enable_cuda=True,
        prefix="/usr/local/osu",
        url="http://mvapich.cse.ohio-state.edu/download/mvapich/osu-micro-benchmarks-5.6.3.tar.gz",
        # with_cuda="/usr/local/cuda",
    )

    # Copy the OSU Micro-Benchmark binaries into the deployment stage
    Stage1 += copy(_from="0", src="/usr/local/osu", dest="/usr/local/osu")

    # Add the OSU Micro-Benchmarks to the default PATH
    base_path = "/usr/local/osu/libexec/osu-micro-benchmarks"
    Stage1 += environment(
        variables={
            "PATH": "{0}:{0}/mpi/collective:{0}/mpi/one-sided:{0}/mpi/pt2pt:{0}/mpi/startup:$PATH".format(
                base_path
            )
        }
    )

Stage0 += comment(f"--- End {filename} ---")



# pylint: disable=invalid-name, too-few-public-methods
# pylint: disable=too-many-instance-attributes

"""OSU benchmarks building block"""

from __future__ import absolute_import
from __future__ import unicode_literals
from __future__ import print_function

import os
import hpccm.templates.wget

from hpccm.building_blocks.base import bb_base
from hpccm.building_blocks.packages import packages
from hpccm.primitives.comment import comment
from hpccm.primitives.copy import copy
from hpccm.primitives.environment import environment
from hpccm.primitives.label import label
from hpccm.primitives.shell import shell
from hpccm.toolchain import toolchain


class osu_benchmarks(
    bb_base,
    hpccm.templates.ConfigureMake,
    hpccm.templates.rm,
    hpccm.templates.tar,
    hpccm.templates.wget,
):
    """OSU benchmarks building block"""

    def __init__(self, **kwargs):
        super(osu_benchmarks, self).__init__()

        self.__ospackages = kwargs.get("ospackages", ["wget", "tar"])
        self.prefix = kwargs.get("prefix", "/usr/local/osu-benchmarks")
        self.__version = kwargs.get("version", "5.4.2")
        self.__toolchain = toolchain(CC="mpicc", CXX="mpicxx")
        self.configure_opts = kwargs.get("configure_opts", [])
        self.__wd = "/var/tmp"  # working directory

        # Filled in by __setup():
        self.__commands = []
        self.__environment_variables = {}
        self.__labels = {}

        self.__setup()

        self.__instructions()

    def __instructions(self):
        self += comment("OSU benchmarks version {}".format(self.__version))
        self += packages(ospackages=self.__ospackages)
        self += shell(commands=self.__commands)
        if self.__environment_variables:
            self += environment(variables=self.__environment_variables)
        if self.__labels:
            self += label(metadata=self.__labels)

    def __setup(self):
        """Construct the series of shell commands, i.e., fill in
        self.__commands"""

        # Get the source
        directory = "osu-micro-benchmarks-{}".format(self.__version)
        tarball = "{}.tar.gz".format(directory)
        self.__commands.append(
            self.download_step(
                url="http://mvapich.cse.ohio-state.edu/download/mvapich/{}".format(
                    tarball
                ),
                directory=self.__wd,
            )
        )
        self.__commands.append(
            self.untar_step(
                tarball=os.path.join(self.__wd, tarball), directory=self.__wd
            )
        )

        # Configure, build, install
        self.__commands.append(
            self.configure_step(
                directory=os.path.join(self.__wd, directory),
                opts=[
                    "CC={}".format(self.__toolchain.CC),
                    "CXX={}".format(self.__toolchain.CXX),
                ],
            )
        )
        self.__commands.append(self.build_step())
        self.__commands.append(self.install_step())

        # Cleanup
        self.__commands.append(
            self.cleanup_step(
                items=[
                    os.path.join(self.__wd, tarball),
                    os.path.join(self.__wd, directory),
                ]
            )
        )

        # Environment
        libexec_path = "{0}/libexec/osu-micro-benchmarks/mpi".format(self.prefix)
        self.__environment_variables[
            "PATH"
        ] = "{0}/collective:{0}/one-sided:{0}/pt2pt:{0}/startup::$PATH".format(
            libexec_path
        )

        # Labels
        self.__labels["osu.version"] = self.__version

    def runtime(self, _from="0"):
        instructions = []
        instructions.append(comment("OSU benchmarks version {}".format(self.__version)))
        instructions.append(copy(_from=_from, src=self.prefix, dest=self.prefix))
        if self.__environment_variables:
            instructions.append(environment(variables=self.__environment_variables))
        if self.__labels:
            instructions.append(label(metadata=self.__labels))
        return "\n".join(str(x) for x in instructions)



#!/usr/bin/env python3
import archspec.cpu
import argparse
import os
import traceback
import sys

import hpccm
from hpccm.building_blocks import (
    packages,
    pip,
)
from hpccm.primitives import baseimage, comment, environment, raw, shell

from ogscm.version import __version__
from ogscm.app import builder
from ogscm.app.deployer import deployer
import shutil


def main():  # pragma: no cover
    recipe_args_parser = argparse.ArgumentParser(add_help=False)
    parser = argparse.ArgumentParser(add_help=False)
    recipe_args_parser.add_argument("recipe", nargs="+")
    parser.add_argument("recipe", nargs="+")

    # General args
    parser.add_argument(
        "--version",
        action="version",
        version="%(prog)s {version}".format(version=__version__),
    )
    parser.add_argument("--out", type=str, default="_out", help="Output directory")
    parser.add_argument(
        "--file", type=str, default="", help="Overwrite output recipe file name"
    )
    parser.add_argument(
        "--print",
        "-P",
        dest="print",
        action="store_true",
        help="Print the definition to stdout",
    )
    general_g = parser.add_argument_group("General image config")
    general_g.add_argument(
        "--format", type=str, choices=["docker", "singularity"], default="docker"
    )
    general_g.add_argument(
        "--base_image",
        type=str,
        default="ubuntu:22.04",
        help="The base image.",
    )
    general_g.add_argument(
        "--runtime_base_image",
        type=str,
        default="",
        help="The runtime base image.",
    )
    general_g.add_argument(
        "--cpu-target",
        type=str,
        # https://github.com/archspec/archspec-json/blob/master/cpu/microarchitectures.json#L94
        # Enables mmx, sse4_2, implemented by CPUs since around 2010.
        default="ivybridge",
        choices=[a for a in sorted(archspec.cpu.TARGETS)],
        help="The CPU microarchitecture to optimize for (archspec).",
    )
    build_g = parser.add_argument_group("Image build options")
    build_g.add_argument(
        "--build",
        "-B",
        dest="build",
        action="store_true",
        help="Build the images from the definition files",
    )
    build_g.add_argument(
        "--build_args",
        type=str,
        default="",
        help="Arguments to the build command. Have to be "
        "quoted and **must** start with a space. E.g. "
        "--build_args ' --no-cache'",
    )
    build_g.add_argument(
        "--upload",
        "-U",
        dest="upload",
        action="store_true",
        help="Upload Docker image to registry",
    )
    build_g.add_argument(
        "--registry",
        type=str,
        default="registry.opengeosys.org/ogs/ogs",
        help="The docker registry the image is tagged and " "uploaded to.",
    )
    build_g.add_argument(
        "--tag",
        type=str,
        default="",
        help="The full docker image tag. Overwrites --registry.",
    )
    build_g.add_argument(
        "--convert",
        "-C",
        dest="convert",
        action="store_true",
        help="Convert Docker image to Singularity image",
    )
    build_g.add_argument(
        "--sif_file",
        type=str,
        default="",
        help="Overwrite output singularity image file name",
    )
    build_g.add_argument(
        "--convert-enroot",
        "-E",
        dest="convert_enroot",
        action="store_true",
        help="Convert Docker image to enroot image",
    )
    build_g.add_argument(
        "--enroot-bundle",
        dest="enroot_bundle",
        action="store_true",
        help="Convert enroot image to enroot bundle",
    )
    build_g.add_argument(
        "--enroot_file",
        type=str,
        default="",
        help="Overwrite output enroot image file name",
    )
    build_g.add_argument(
        "--force",
        dest="force",
        action="store_true",
        help="Forces overwriting of image files!",
    )
    build_g.add_argument(
        "--runtime-only",
        "-R",
        dest="runtime_only",
        action="store_true",
        help="Generate multi-stage Dockerfiles for small runtime " "images",
    )
    maint_g = parser.add_argument_group("Maintenance")
    maint_g.add_argument(
        "--clean",
        dest="cleanup",
        action="store_true",
        help="Cleans up generated files in default directories.",
    )
    deploy_g = parser.add_argument_group("Image deployment")
    deploy_g.add_argument(
        "--deploy",
        "-D",
        nargs="?",
        const="ALL",
        type=str,
        default="",
        help="Deploys to all configured hosts (in config/deploy_hosts.yml) with no additional arguments or to the specified host. Implies --build and --convert arguments.",
    )

    install_g = parser.add_argument_group("Packages to install")
    install_g.add_argument(
        "--pip",
        nargs="*",
        type=str,
        default=[],
        metavar="package",
        help="Install additional Python packages",
    )
    install_g.add_argument(
        "--packages",
        nargs="*",
        type=str,
        default=[],
        metavar="packages",
        help="Install additional OS packages",
    )

    args = parser.parse_known_args()[0]

    images_out_dir = os.path.abspath(f"{args.out}/images")
    if not os.path.exists(images_out_dir):
        os.makedirs(images_out_dir)

    hpccm.config.set_cpu_target(args.cpu_target)
    Stage0 = hpccm.Stage()
    Stage0 += raw(docker="# syntax=docker/dockerfile:1.4")

    if args.runtime_only:
        Stage0.name = "build"
    Stage0 += baseimage(image=args.base_image, _as="build", _arch="x86_64")

    Stage0 += comment(
        f"Generated with ogs-container-maker {__version__}", reformat=False
    )
    Stage0 += packages(
        ospackages=["wget", "tar", "curl", "ca-certificates", "make", "unzip"]
    )

    # Prepare runtime stage
    Stage1 = hpccm.Stage()
    if args.runtime_base_image == "":
        Stage1.baseimage(image=args.base_image)
    else:
        Stage1.baseimage(image=args.runtime_base_image)
        if "jupyter/base-notebook-ubuntu" in args.runtime_base_image:
            Stage1 += raw(docker="USER root")

    # set timezone
    packages_tz = packages(ospackages=["tzdata"])
    env_tz = environment(variables={"TZ": "Europe/Berlin"})
    shell_tz = shell(
        commands=[
            "ln -snf /usr/share/zoneinfo/$TZ /etc/localtime",
            "dpkg-reconfigure -f noninteractive tzdata",
        ]
    )
    Stage0 += [packages_tz, env_tz, shell_tz]
    Stage1 += [packages_tz, env_tz, shell_tz]

    cwd = os.getcwd()
    img_file = ""
    out_dir = f"{args.out}/{args.format}"
    toolchain = None
    versions = None  # versions.json
    pip_requirements = []

    for recipe in recipe_args_parser.parse_known_args()[0].recipe:
        import importlib.resources as pkg_resources
        from ogscm import recipes

        # https://stackoverflow.com/a/1463370/80480
        ldict = {"filename": recipe}
        try:
            recipe_builtin = pkg_resources.read_text(recipes, recipe)
            exec(compile(recipe_builtin, recipe, "exec"), locals(), ldict)
        except Exception as err:
            error_class = err.__class__.__name__
            detail = err.args[0]
            cl, exc, tb = sys.exc_info()
            line_number = traceback.extract_tb(tb)[-1][1]
            print(f"{error_class} at line {line_number}: {detail}")
            if not os.path.exists(recipe):
                print(f"{recipe} does not exist!")
                exit(1)

            with open(recipe, "r") as reader:
                exec(compile(reader.read(), recipe, "exec"), locals(), ldict)
        if "out_dir" in ldict:
            out_dir = ldict["out_dir"]
        if "toolchain" in ldict:
            toolchain = ldict["toolchain"]
        if "img_file" not in ldict:
            print(f"img_file variable has to be set in {recipe}!")
            exit(1)
        img_file = ldict["img_file"]
        if "versions" in ldict:
            versions = ldict["versions"]
        if "pip_requirements" in ldict:
            pip_requirements = ldict["pip_requirements"]

    # Workaround to get the full help message
    help_parser = argparse.ArgumentParser(
        parents=[parser], formatter_class=argparse.ArgumentDefaultsHelpFormatter
    )
    help_parser.parse_args()

    # Finally parse
    args = parser.parse_args()

    ### container_info ###
    definition_file = "Dockerfile"
    if args.format == "singularity":
        definition_file = "Singularity.def"
    definition_file_path = os.path.join(out_dir, definition_file)
    if img_file[0] == "-":
        img_file = img_file[1:]
    if args.tag != "":
        tag = args.tag
    else:
        tag = f"{args.registry}/{img_file.lower()}:latest"
    # TODO:
    # context_path_size = len(self.ogsdir)
    # "{self.out_dir[context_path_size+1:]}/{self.definition_file}"
    ### end container_info ###

    if args.cleanup:
        shutil.rmtree(out_dir, ignore_errors=True)
        print("Cleaned up!")
        exit(0)
    if not os.path.exists(out_dir):
        os.makedirs(out_dir)  # For .scif files

    # General args
    if args.packages:
        Stage0 += packages(ospackages=args.packages)
        Stage1 += packages(ospackages=args.packages)

    if args.pip:
        Stage0 += pip(packages=args.pip, pip="pip3")
        Stage1 += pip(packages=args.pip, pip="pip3")

    # Create definition
    hpccm.config.set_container_format(args.format)
    hpccm.config.set_singularity_version("3.5")

    stages_string = str(Stage0)

    if args.runtime_only:
        runtime_exclude = []
        if hasattr(args, "mfront") and not args.mfront:
            runtime_exclude.append("boost")
        Stage1 += Stage0.runtime(exclude=runtime_exclude)
        if (
            hasattr(args, "compiler")
            and args.compiler == "gcc"
            and args.compiler_version != None
        ):
            Stage1 += packages(apt=["libstdc++6"])
        if "jupyter/base-notebook-ubuntu" in args.runtime_base_image:
            Stage1 += raw(docker="USER ${NB_USER}")
        stages_string += "\n\n" + str(Stage1)

    # ---------------------------- recipe end -----------------------------
    with open(definition_file_path, "w") as f:
        print(stages_string, file=f)
    if args.print:
        print(stages_string)
    else:
        print(f"Created definition {os.path.abspath(definition_file_path)}")

    # Create image
    if not args.build:
        exit(0)

    b = builder(args, images_out_dir, img_file, definition_file_path, tag, cwd)
    b.build()

    # Deploy image
    if not args.deploy:
        exit(0)

    deployer(args.deploy, cwd, b.image_file)


if __name__ == "__main__":  # pragma: no cover
    main()



from hpccm.building_blocks import pip
from hpccm.primitives import comment

print(f"Evaluating {filename}")

# Add cli arguments to args_parser
parse_g = parser.add_argument_group(filename)
parse_g.add_argument("--vtk_version", type=str, default="8.1.2")

# Parse local args
local_args = parser.parse_known_args()[0]

# set image file name
img_file = f"vtk-{local_args.vtk_version}"

# Optionally set out_dir
out_dir = f"{local_args.out}/{local_args.format}/vtk/{local_args.vtk_version}"

# Implement recipe
Stage0 += comment(f"Begin {filename}")

Stage0 += pip(pip="pip3", packages=[f"vtk=={local_args.vtk_version}"])

Stage0 += comment(f"--- End {filename} ---")



# Recipe template
# Have a look and change statements with ### SET comment

### SET imports
from hpccm.primitives import comment, shell, raw
from hpccm.building_blocks import packages

print(f"Evaluating {filename}")

# Add cli arguments to args_parser
parse_g = parser.add_argument_group(filename)

### SET arguments, e.g:
# parse_g.add_argument("--my_arg", type=str, default="default_value")

# Parse local args
local_args = parser.parse_known_args()[0]

### SET append to image file name, e.g.:
img_file += f"-web"

### SET Append to out_dir, e.g.:
out_dir += f"/web"

# Implement recipe
Stage0 += comment(f"Begin {filename}")

Stage0 += shell(commands=["curl -fsSL https://deb.nodesource.com/setup_18.x | bash -"])
Stage0 += packages(ospackages=["nodejs", "pandoc"])
Stage0 += shell(
    commands=[
        "npm install --global yarn",
        "wget https://github.com/gohugoio/hugo/releases/download/v0.101.0/hugo_0.101.0_Linux-64bit.deb",
        "dpkg -i hugo_0.101.0_Linux-64bit.deb",
        "rm hugo_0.101.0_Linux-64bit.deb",
    ]
)

Stage0 += comment(f"--- End {filename} ---")



from __future__ import absolute_import

from ogscm.building_blocks.ccache import ccache
from ogscm.building_blocks.lmod import lmod
from ogscm.building_blocks.ogs import ogs
from ogscm.building_blocks.ogs_base import ogs_base
from ogscm.building_blocks.osu_benchmarks import osu_benchmarks
from ogscm.building_blocks.paraview import paraview
from ogscm.building_blocks.pm_easybuild import pm_easybuild
